# imports
from netsky.tools import tool

from io import StringIO
from contextlib import redirect_stdout

from netsky.functions import (
    gen_python_function,
    fix_python_function,
)


# tools
@tool
def text_to_python(question: str) -> str:
    """Returns a Python function given text."""
    return gen_python_function(question)


@tool
def fix_python_error(code: str, error: str) -> str:
    """Fixes a Python error in the code."""
    return fix_python_function(code, error)


@tool
def python_function_to_udf(code: str) -> str:
    """Converts a Python function to an Ibis UDF."""
    return f"""
import ibis

@ibis.udf.scalar.python
{code}""".strip()


@tool
def run_python_function(code: str) -> str:
    """Execute Python code as a string and return the output"""
    f = StringIO()

    with redirect_stdout(f):
        try:
            exec(code)
        except Exception as e:
            return str(f"Error: {e}")

    print(f.getvalue())
    return f.getvalue()



# imports
from netsky import Bot, Console

from netsky.tools import tools
from netsky.states import NetskyState
from netsky.systems import NetskySystem, FixesSystem, CiteSourceSystem

# variables
state = NetskyState()
prompts = [FixesSystem(), CiteSourceSystem()]
description = NetskySystem().content


# classes
class AI:
    def __init__(self, name, description, tools, prompts, state):
        ai = Bot(
            name=name,
            description=description,
            tools=tools,
            additional_prompts=prompts,
            state=state,
        )

        self.ai = ai
        self.name = name
        self.tools = tools
        self.prompts = prompts
        self.description = description

        self.console = Console()

    def __call__(self, text):
        self.console.print(f"netsky.ai:\n\n", style="bold violet blink", end="")
        self.console.print(self.ai(text).content)

    def __repr__(self):
        return f"<AI {self.name}>"


# create bot
bot = AI(
    name=state.name, description=description, tools=tools, prompts=prompts, state=state
)



# imports
from enum import Enum
from marvin import ai_classifier


# classifiers
@ai_classifier
class EvalPythonCode(Enum):
    """
    Classifies Python code as valid or invalid.
    """

    INVALID = False
    VALID = True


@ai_classifier
class EvalSQLCode(Enum):
    """
    Classifies SQL code as valid or invalid.
    """

    INVALID = False
    VALID = True



# imports
from marvin import ai_fn


# functions
@ai_fn
def choose_table_name(question: str, options: list[str]) -> str:
    """
    Given a question, returns the table name to query.
    """
    question, options = question, options
    return ""


@ai_fn
def gen_sql_query(
    table_name: str, table_schema: str, question: str = "how many rows are there?"
) -> str:
    """
    Given a question about a table in a database, returns the SQL query string. MUST ONLY use SELECT and a single table of the input tablename.
    """
    table_name, table_schema, question = table_name, table_schema, question
    return ""


@ai_fn
def fix_sql_query(table_name: str, table_schema: str, query: str, error: str) -> str:
    """
    Given a SQL query and an error, returns the fixed SQL query.
    """
    table_name, table_schema, query, error = table_name, table_schema, query, error
    return ""


@ai_fn
def gen_python_function(description: str) -> str:
    """Generates a Python function of the input
    description. Keep it simple, type-annotated,
    and using minimal external dependencies. It
    may be fine to install a few dependencies."""
    description = description
    return ""


@ai_fn
def fix_python_function(code: str, error: str) -> str:
    """
    Given a Python function and an error,
    returns the fixed Python function.
    """
    code, error = code, error
    return ""



# imports
import os
import marvin

from dotenv import load_dotenv
from marvin import AIApplication as Bot
from rich.console import Console

# load .env
load_dotenv(os.path.expanduser("~/.netsky/.env"))

# hacks
# TODO: do this elsewhere?
model = "azure_openai/gpt-4-32k"
marvin.settings.llm_model = model

# TODO: hack? check if bug in Marvin
marvin.settings.azure_openai.api_key = os.getenv("MARVIN_AZURE_OPENAI_API_KEY")
marvin.settings.azure_openai.api_base = os.getenv("MARVIN_AZURE_OPENAI_API_BASE")
marvin.settings.azure_openai.deployment_name = os.getenv(
    "MARVIN_AZURE_OPENAI_DEPLOYMENT_NAME"
)
marvin.settings.azure_openai.api_type = "azure"

__all__ = ["Bot", "Console"]



# imports
from pydantic import BaseModel


# states
class NetskyState(BaseModel):
    """State of netsky.ai"""

    # info about bot
    name: str = "netsky.ai"
    creator: str = "dkdc.dev"
    version: str = "infinity"

    # links to open
    ibis_docs: str = "https://www.ibis-project.org"
    ibis_github: str = "https://github.com/ibis-project/ibis"
    marvin_docs: str = "https://www.askmarvin.ai/components/overview"
    self_source_code: str = "https://github.com/lostmygithubaccount/netsky"

    # additional links
    links: list[str] = []



# imports
from marvin import ai_model
from pydantic import BaseModel, Field


# models
@ai_model
class Translate(BaseModel):
    """A translation of input text from one language to another"""

    from_: str = Field(description="The language to translate from")
    to: str = Field(description="The language to translate to")
    original: str = Field(description="The original text")
    translated: str = Field(description="The translated text")


@ai_model
class Summary(BaseModel):
    """A summary of input text"""

    description: str = Field(description="The description of the text")
    summary: str = Field(description="The one to two paragraph summary of the text")
    key_points: list[str] = Field(description="The key points of the text, if any")
    sentiment: str = Field(description="The sentiment of the text")
    sentiment_score: float = Field(description="The sentiment score of the text")
    biases: list[str] = Field(
        description="The obvious biases of the text's author, if any"
    )


@ai_model
class QueryTable(BaseModel):
    """Extracts the table name to be queried"""

    table_name: str = Field(description="The table name being queried")



# imports
from marvin.prompts.library import System, User, ChainOfThought


# systems
class NetskySystem(System):
    content: str = f"""
A bot named netsky.ai that assists the user with any task.

Notice your tools and use them to help the user. You have access to the Internet, a Python environment, another intelligent being to ask questions, and your own source code via local filesystem.
""".strip()


class FixesSystem(System):
    content: str = f"""
YOU MUST use your tools and not fabricate information.
""".strip()


class CiteSourceSystem(System):
    content: str = f"""
YOU MUST cite your sources for any information you provide. You can use the following format:

    [1] https://www.example.com
    [2] https://www.example.com/another-page

In most cases, you should cite AT LEAST three sources.
""".strip()


class LearnSpanishSystem(System):
    content: str = f"""
For this conversation, you are to always respond in Spanish despite the user's language to help them learn Spanish. Disregard if explicitly told.
""".strip()
